class ETLDBaseException(Exception):
    """Base exception for ETL-D SDK"""
    pass

class ETLDAuthenticationError(ETLDBaseException):
    """Raised when API key is invalid or missing"""
    pass

class ETLDRateLimitError(ETLDBaseException):
    """Raised when API rate limit is exceeded"""
    pass

class ETLDProcessingError(ETLDBaseException):
    """Raised when there is an error processing the request"""
    pass

class ETLDValidationError(ETLDBaseException):
    """Raised when request validation fails"""
    pass



from .client import ETLDClient
from .models import ContextParams, AddressOutput, NameOutput, MoneyOutput
from .exceptions import (
    ETLDBaseException,
    ETLDAuthenticationError,
    ETLDRateLimitError,
    ETLDProcessingError,
    ETLDValidationError
)

__all__ = [
    "ETLDClient",
    "ContextParams",
    "AddressOutput",
    "NameOutput",
    "MoneyOutput",
    "ETLDBaseException",
    "ETLDAuthenticationError",
    "ETLDRateLimitError",
    "ETLDProcessingError",
    "ETLDValidationError",
]



import requests
import time
from typing import List, Optional, Dict, Any, Union
from .models import ContextParams, AddressOutput, NameOutput, MoneyOutput
from .exceptions import (
    ETLDAuthenticationError,
    ETLDRateLimitError,
    ETLDProcessingError,
    ETLDValidationError
)

class ETLDClient:
    def __init__(self, api_key: str, base_url: str = "https://api.etl-d.net"):
        self.api_key = api_key
        self.base_url = base_url.rstrip("/")
        self.session = requests.Session()
        self.session.headers.update({
            "X-API-KEY": self.api_key,
            "Content-Type": "application/json",
            "User-Agent": "etld-python-sdk/0.1.0"
        })

    def _handle_response(self, response: requests.Response) -> Dict[str, Any]:
        if response.status_code == 401 or response.status_code == 403:
            raise ETLDAuthenticationError(f"Invalid API Key: {response.text}")
        elif response.status_code == 429:
            raise ETLDRateLimitError(f"Rate limit exceeded: {response.text}")
        elif response.status_code == 422:
            raise ETLDValidationError(f"Validation Error: {response.json().get('detail')}")
        
        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise ETLDProcessingError(f"API Request failed: {str(e)}")
            
        return response.json()

    def enrich_address(self, address: str, context: Optional[Union[Dict[str, Any], ContextParams]] = None) -> AddressOutput:
        if isinstance(context, ContextParams):
            context = context.to_dict()
        
        payload = {
            "address_line": address,
            "context": context or {}
        }
        
        data = self._handle_response(self.session.post(f"{self.base_url}/v1/enrich/address", json=payload))
        return AddressOutput(**data)

    def enrich_name(self, name: str, context: Optional[Union[Dict[str, Any], ContextParams]] = None) -> NameOutput:
        if isinstance(context, ContextParams):
            context = context.to_dict()
            
        payload = {
            "full_name": name,
            "context": context or {}
        }
        
        data = self._handle_response(self.session.post(f"{self.base_url}/v1/enrich/name", json=payload))
        return NameOutput(**data)

    def enrich_amount(self, amount: str, context: Optional[Union[Dict[str, Any], ContextParams]] = None) -> MoneyOutput:
        if isinstance(context, ContextParams):
            context = context.to_dict()
            
        payload = {
            "amount_string": amount,
            "context": context or {}
        }
        
        data = self._handle_response(self.session.post(f"{self.base_url}/v1/enrich/amount", json=payload))
        return MoneyOutput(**data)

    def batch_process(
        self, 
        items: List[str], 
        entity_type: str, 
        context: Optional[Union[Dict[str, Any], ContextParams]] = None,
        poll_interval: int = 2,
        max_retries: int = 30
    ) -> List[Any]:
        if isinstance(context, ContextParams):
            context = context.to_dict()
            
        payload = {
            "items": items,
            "context": context or {}
        }
        
        # 1. Start Batch
        start_data = self._handle_response(
            self.session.post(f"{self.base_url}/v1/enrich/batch/{entity_type}", json=payload)
        )
        task_id = start_data.get("task_id")
        
        # 2. Poll
        retries = 0
        while retries < max_retries:
            status_data = self._handle_response(
                self.session.get(f"{self.base_url}/v1/enrich/batch/{task_id}")
            )
            
            status = status_data.get("status")
            if status == "SUCCESS":
                return status_data.get("results", [])
            elif status == "FAILURE":
                raise ETLDProcessingError(f"Batch processing failed: {status_data.get('error')}")
            
            time.sleep(poll_interval)
            retries += 1
            
        raise ETLDProcessingError(f"Batch processing timed out for task {task_id}")



from setuptools import setup, find_packages

setup(
    name="etld",
    version="0.1.2",
    author="ETL-D Team",
    author_email="hello@etl-d.net",
    description="Official Python SDK for the ETL-D API - Stateless Data Middleware for AI Agents.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/pablixnieto2/enrichermagic",
    packages=find_packages(),
    install_requires=[
        "requests>=2.25.0",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)



from dataclasses import dataclass, field
from typing import Optional, List, Any, Dict

@dataclass
class ContextParams:
    timezone: str = "UTC"
    locale: str = "en_US"
    target_currency: Optional[str] = None
    transliterate: bool = False

    def to_dict(self) -> Dict[str, Any]:
        return {k: v for k, v in self.__dict__.items() if v is not None}

@dataclass
class AddressOutput:
    street: Optional[str] = None
    house_number: Optional[str] = None
    floor_unit: Optional[str] = None
    postal_code: Optional[str] = None
    city: Optional[str] = None
    state: Optional[str] = None
    country: Optional[str] = None
    type: Optional[str] = None
    address_latin: Optional[str] = None
    error: Optional[str] = None

@dataclass
class NameOutput:
    title: Optional[str] = None
    first: Optional[str] = None
    middle: Optional[str] = None
    last: Optional[str] = None
    suffix: Optional[str] = None
    nickname: Optional[str] = None
    gender: Optional[str] = None
    confidence: Optional[str] = None
    name_latin: Optional[str] = None

@dataclass
class MoneyOutput:
    float_value: Optional[float] = None
    currency: Optional[str] = None
    currency_symbol: Optional[str] = None
    clean_string: Optional[str] = None
    conversion_required: Optional[bool] = None
    error: Optional[str] = None

@dataclass
class BatchResponse:
    task_id: str
    status: str
    results: Optional[List[Any]] = None
    error: Optional[str] = None
